#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

#include <ysclass.h>

#ifdef WIN32
#include <windows.h>
#endif

#ifndef __APPLE__
#include <GL/gl.h>
#include <GL/glu.h>
#else
#include <OpenGL/gl.h>
#include <OpenGL/glu.h>
#endif

#include <fssimplewindow.h>
#include <fsgui.h>


static YSBOOL terminateApp=YSFALSE;



class FsApplicationMenu : public FsGuiPopUpMenu
{
public:
	FsGuiPopUpMenuItem *fileMenu;
	FsGuiPopUpMenuItem *fileOpen,*fileSave,*fileExit;

	FsApplicationMenu();
	void MakeMenu(void);

	virtual void OnSelectMenuItem(FsGuiPopUpMenuItem *item);
};

FsApplicationMenu::FsApplicationMenu()
{
	::terminateApp=YSFALSE;
}

void FsApplicationMenu::MakeMenu(void)
{
	this->SetIsPullDownMenu(YSTRUE);

	fileMenu=AddTextItem(0,FSKEY_F,"File (F)");
	fileOpen=fileMenu->GetSubMenu()->AddTextItem(0,FSKEY_O,"Open (O)");
	fileSave=fileMenu->GetSubMenu()->AddTextItem(0,FSKEY_S,"Save (S)");
	fileExit=fileMenu->GetSubMenu()->AddTextItem(0,FSKEY_X,"Exit (X)");
}

void FsApplicationMenu::OnSelectMenuItem(FsGuiPopUpMenuItem *item)
{
	if(item==fileExit)
	{
		::terminateApp=YSTRUE;
	}
}



////////////////////////////////////////////////////////////



class FsApplicationDialog : public FsGuiDialog
{
public:
	FsGuiButton *okBtn;
	FsGuiButton *cancelBtn;

	FsApplicationDialog();
	void MakeDialog(void);
	virtual void OnButtonClick(FsGuiButton *btn);
};

FsApplicationDialog::FsApplicationDialog()
{
	terminateApp=YSFALSE;
}

void FsApplicationDialog::MakeDialog(void)
{
	SetTopLeftCorner(0,32);

	AddStaticText(0,FSKEY_NULL,"Sample Dialog",16,1,YSTRUE);
	okBtn=AddTextButton(0,FSKEY_NULL,FSGUI_PUSHBUTTON,"OK",YSTRUE);
	cancelBtn=AddTextButton(0,FSKEY_NULL,FSGUI_PUSHBUTTON,"Cancel",YSTRUE);

	SetTransparency(YSFALSE);

	Fit();
}

void FsApplicationDialog::OnButtonClick(FsGuiButton *btn)
{
	if(btn==okBtn || btn==cancelBtn)
	{
		terminateApp=YSTRUE;
	}
}



////////////////////////////////////////////////////////////



class FsApplicationCanvas : public FsGuiCanvas
{
};



static void MainLoopOneStep(void);
static FsApplicationMenu *mainMenu=NULL;
static FsApplicationDialog *mainDlg=NULL;
static FsApplicationCanvas *mainCanvas=NULL;
static const int sleepTime=20;


void FsGuiAppInitialize(int ac,char *av[])
{
	srand((unsigned int)time(NULL));

	FsOpenWindow(16,16,1024,768,1);



	FsGuiDefaultAsciiFixedFontRenderer fixedFontRenderer;
	FsGuiObject::defAsciiRenderer=&fixedFontRenderer;
	FsGuiObject::defUnicodeRenderer=&fixedFontRenderer;


	mainMenu=new FsApplicationMenu;
	mainMenu->MakeMenu();

	mainDlg=new FsApplicationDialog;
	mainDlg->MakeDialog();

	mainCanvas=new FsApplicationCanvas;
	mainCanvas->SetMainMenu(mainMenu);
	mainCanvas->AddDialog(mainDlg);
}

void FsGuiAppInterval(void)
{
	FsPollDevice();


	mainCanvas->Interval();


	int wid,hei;
	FsGetWindowSize(wid,hei);


	const YSBOOL shift=(YSBOOL)FsGetKeyState(FSKEY_SHIFT);
	const YSBOOL ctrl=(YSBOOL)FsGetKeyState(FSKEY_CTRL);
	const YSBOOL alt=(YSBOOL)FsGetKeyState(FSKEY_ALT);

	int key;
	while(key=FsInkey())
	{
		mainCanvas->KeyIn(key,shift,ctrl,alt);
	}
	while(key=FsInkeyChar())
	{
		mainCanvas->CharIn(key);
	}

	int lb,mb,rb,mx,my;
	int mouseEvent;
	while(FSMOUSEEVENT_NONE!=(mouseEvent=FsGetMouseEvent(lb,mb,rb,mx,my)))
	{
		mainCanvas->SetMouseState((YSBOOL)lb,(YSBOOL)mb,(YSBOOL)rb,mx,my);
	}



	if(0!=FsCheckWindowExposure() || YSTRUE==mainCanvas->NeedRedraw())
	{
		glClearColor(1.0,1.0,1.0,1.0);
		glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

		mainCanvas->Show();

		glFlush();
		FsSwapBuffers();
	}

	FsSleep(sleepTime);
}

int FsGuiAppNeedToTerminate(void)
{
	return (int)terminateApp;
}

void FsGuiAppCleanUp(void)
{
	delete mainCanvas;
	delete mainMenu;
	delete mainDlg;

}
