#include "ysfixedfontrenderer.h"
#include <ysglfontdata.h>

YsFixedFontRenderer::YsFixedFontRenderer()
{
	Initialize();
}

YsFixedFontRenderer::~YsFixedFontRenderer()
{
}

void YsFixedFontRenderer::Initialize(void)
{
	fontWid=12;
	fontHei=16;
	fontPtr=YsFont12x16;
}

int YsFixedFontRenderer::GetFontWidth(void) const
{
	return fontWid;
}

int YsFixedFontRenderer::GetFontHeight(void) const
{
	return fontHei;
}

YSRESULT YsFixedFontRenderer::RequestDefaultFontWithPixelHeight(unsigned int requestedHei)
{
	int fontWid,fontHei;
	const unsigned char *const *fontPtr=YsGlSelectFontBitmapPointerByHeight(&fontWid,&fontHei,requestedHei);
	if(NULL!=fontPtr)
	{
		this->fontWid=fontWid;
		this->fontHei=fontHei;
		this->fontPtr=fontPtr;
		return YSOK;
	}
	return YSERR;
}

YSRESULT YsFixedFontRenderer::RenderString(YsBitmap &bmp,const wchar_t wStr[],const YsColor &fgCol,const YsColor &bgCol) const
{
	int nWid,nHei;
	CountStringDimension(nWid,nHei,wStr);

	YsArray <char> cStrBuf(nWid+1,NULL);

	bmp.PrepareBitmap(nWid*GetFontWidth(),nHei*GetFontHeight());
	bmp.Clear((unsigned char)bgCol.Ri(),(unsigned char)bgCol.Gi(),(unsigned char)bgCol.Bi(),0);

	int i=0,i0=0,cStrPtr=0,y=bmp.GetHeight()-GetFontHeight();
	for(;;)
	{
		if(0==wStr[i] || '\n'==wStr[i])
		{
			cStrBuf[cStrPtr]=0;
			YsGlWriteStringToRGBA8Bitmap(
			    cStrBuf,fgCol.Ri(),fgCol.Gi(),fgCol.Bi(),255,
			    bmp.GetEditableRGBABitmapPointer(),bmp.GetWidth(),bmp.GetHeight(),
			    0,y,
			    fontPtr,fontWid,fontHei);
			y-=fontHei;
			i0=i+1;
			cStrPtr=0;
		}
		else
		{
			cStrBuf[cStrPtr++]=(char)wStr[i];
		}

		if(0==wStr[i])
		{
			break;
		}
		i++;
	}

	return YSOK;
}

YSRESULT YsFixedFontRenderer::GetTightRenderSize(int &wid,int &hei,const wchar_t wStr[]) const
{
	int nWid,nHei;
	CountStringDimension(nWid,nHei,wStr);
	wid=fontWid*nWid;
	hei=fontHei*nHei;
	return YSOK;
}

