/* ////////////////////////////////////////////////////////////

  File Name: configmenu.cpp
  Copyright (c) 2014 Soji Yamakawa.  All rights reserved.
  http://www.ysflight.com

//////////////////////////////////////////////////////////// */

#include <ysport.h>
#include <ysgl.h>
#include <fsguifiledialog.h>

#include "../fsgui3dapp.h"
#include "../miscdlg.h"
#include "../filename.h"
#include "configmenu.h"
#include "config.h"

#include "../textresource.h"

PolyCreConfigDialog::PolyCreConfigDialog()
{
	mbox=new FsGuiMessageBoxDialog;
	fdlg=new FsGuiFileDialog;
}

PolyCreConfigDialog::~PolyCreConfigDialog()
{
	if(NULL!=mbox)
	{
		delete mbox;
		mbox=NULL;
	}
	if(NULL!=fdlg)
	{
		delete fdlg;
		fdlg=NULL;
	}
}

/*static*/ PolyCreConfigDialog *PolyCreConfigDialog::Create(void)
{
	return new PolyCreConfigDialog;
}

/*static*/ void PolyCreConfigDialog::Delete(PolyCreConfigDialog *ptr)
{
	delete ptr;
}

void PolyCreConfigDialog::Make(class PolyCreConfig &cfg,class FsGui3DMainCanvas *canvas)
{
	this->canvas=canvas;

	FsGuiDialog::Initialize();

	SetArrangeType(FSDIALOG_ARRANGE_TOP_LEFT);

	const wchar_t * const coordSysLabel[3]=
	{
		FSGUI_DLG_CONFIG_RIGHTHANDCOORD,
		FSGUI_DLG_CONFIG_LEFTHANDCOORD,
		NULL
	};

	coordSysDrp=AddDropList(0,FSKEY_NULL,"Coordinate System",2,coordSysLabel,2,40,40,YSTRUE);
	if(YSOPENGL==cfg.coordSys)
	{
		coordSysDrp->Select(0);
	}
	else
	{
		coordSysDrp->Select(1);
	}

	setDefDataPathBtn=AddTextButton(0,FSKEY_NULL,FSGUI_PUSHBUTTON,L"Set",YSTRUE);
	AddStaticText(0,FSKEY_NULL,L"Default Data Path=",YSFALSE);
	defDataPath=AddStaticText(0,FSKEY_NULL,cfg.defDataDir,YSFALSE);

	bgColorPlt[0]=AddColorPalette(0,FSKEY_NULL,"",cfg.bgColor[0].Ri(),cfg.bgColor[0].Gi(),cfg.bgColor[0].Bi(),8,YSTRUE,YSTRUE);
	bgColorPlt[1]=AddColorPalette(0,FSKEY_NULL,"",cfg.bgColor[1].Ri(),cfg.bgColor[1].Gi(),cfg.bgColor[1].Bi(),8,YSTRUE,YSFALSE);
	bgColorPlt[2]=AddColorPalette(0,FSKEY_NULL,"",cfg.bgColor[2].Ri(),cfg.bgColor[2].Gi(),cfg.bgColor[2].Bi(),8,YSTRUE,YSFALSE);

	okBtn=AddTextButton(0,FSKEY_NULL,FSGUI_PUSHBUTTON,FSGUI_COMMON_OK,YSTRUE);
	cancelBtn=AddTextButton(0,FSKEY_ESC,FSGUI_PUSHBUTTON,FSGUI_COMMON_CANCEL,YSFALSE);

	setDefaultBtn=AddTextButton(0,FSKEY_NULL,FSGUI_PUSHBUTTON,FSGUI_DLG_CONFIG_SETDEFAULT,YSFALSE);

	SetTransparency(YSFALSE);
	Fit();
}

void PolyCreConfigDialog::SetFromConfig(class PolyCreConfig &cfg)
{
	if(YSOPENGL==cfg.coordSys)
	{
		coordSysDrp->Select(0);
	}
	else
	{
		coordSysDrp->Select(1);
	}

	defDataPath->SetText(cfg.defDataDir);

	bgColorPlt[0]->SetColor(cfg.bgColor[0]);
	bgColorPlt[1]->SetColor(cfg.bgColor[1]);
	bgColorPlt[2]->SetColor(cfg.bgColor[2]);

}

void PolyCreConfigDialog::Retrieve(class PolyCreConfig &cfg)
{
	switch(coordSysDrp->GetSelection())
	{
	case 0:
		cfg.coordSys=YSRIGHT_ZMINUS_YPLUS;
		break;
	case 1:
		cfg.coordSys=YSLEFT_ZPLUS_YPLUS;
		break;
	}

	defDataPath->GetText(cfg.defDataDir);

	bgColorPlt[0]->GetColor(cfg.bgColor[0]);
	bgColorPlt[1]->GetColor(cfg.bgColor[1]);
	bgColorPlt[2]->GetColor(cfg.bgColor[2]);
}

void PolyCreConfigDialog::OnButtonClick(FsGuiButton *btn)
{
	if(okBtn==btn)
	{
		PolyCreConfig *cfg=PolyCreConfig::Create();
		Retrieve(*cfg);

		const wchar_t *ysflightComDir=canvas->fileAndDirName->GetYsflightComDir();
		YsFileIO::MkDir(ysflightComDir);

		const wchar_t *appDataDir=canvas->fileAndDirName->GetAppDataDir();
		YsFileIO::MkDir(appDataDir);

		const wchar_t *configDir=canvas->fileAndDirName->GetOptionDir();
		YsFileIO::MkDir(configDir);

		const wchar_t *configFileName=canvas->fileAndDirName->GetOptionFileName();
		cfg->Save(configFileName);

		canvas->Option_ReapplyConfig(*cfg);

		PolyCreConfig::Delete(cfg);

		CloseModalDialog(YSOK);
	}
	else if(cancelBtn==btn)
	{
		CloseModalDialog(YSERR);
	}
	else if(setDefDataPathBtn==btn)
	{
		mbox->Make(L"",L"Select a .SRF file in the default data directory.",L"Close",NULL,0,0);
		mbox->SetCloseModalCallBack(BeginSelectDefaultDataDirectory);
		AttachModalDialog(mbox);
	}
	else if(setDefaultBtn==btn)
	{
		auto *cfg=PolyCreConfig::Create();
		SetFromConfig(*cfg);
		PolyCreConfig::Delete(cfg);
	}
}

/*static*/ void PolyCreConfigDialog::BeginSelectDefaultDataDirectory(FsGuiControlBase *dlg,FsGuiDialog *,int)
{
	PolyCreConfigDialog *configDlg=(PolyCreConfigDialog *)dlg;
	FsGui3DMainCanvas *canvas=configDlg->canvas;

	YsString dataDir;
	dataDir.EncodeUTF8 <wchar_t> (canvas->GetDataDir());

	YsString def;
	def.MakeFullPathName(dataDir,"*.srf");

	configDlg->fdlg->Initialize();
	configDlg->fdlg->mode=FsGuiFileDialog::MODE_OPEN;
	configDlg->fdlg->multiSelect=YSFALSE;
	configDlg->fdlg->title.Set(L"Open");
	configDlg->fdlg->fileExtensionArray.Append(L".srf");
	configDlg->fdlg->defaultFileName.SetUTF8String(def);
	configDlg->fdlg->SetCloseModalCallBack(SelectDefaultDataDirectory_FileSelected);
	configDlg->AttachModalDialog(configDlg->fdlg);
}

/*static*/ void PolyCreConfigDialog::SelectDefaultDataDirectory_FileSelected(FsGuiControlBase *configDlgIn,FsGuiDialog *fdlgIn,int returnValue)
{
	if(YSOK==(YSRESULT)returnValue)
	{
		PolyCreConfigDialog *configDlg=(PolyCreConfigDialog *)configDlgIn;
		FsGuiFileDialog *fdlg=(FsGuiFileDialog *)fdlgIn;

		YsWString fn=fdlg->selectedFileArray[0];
		YsWString pth,fil;
		fn.SeparatePathFile(pth,fil);

		configDlg->defDataPath->SetText(pth);
	}
}

////////////////////////////////////////////////////////////

/*static*/ void FsGui3DMainCanvas::Option_Config(void *appPtr,FsGuiPopUpMenu *,FsGuiPopUpMenuItem *)
{
	FsGui3DMainCanvas *canvas=(FsGui3DMainCanvas *)appPtr;

	if(NULL==canvas->configDlg)
	{
		canvas->configDlg=PolyCreConfigDialog::Create();
	}

	PolyCreConfig *cfg=PolyCreConfig::Create();

	const wchar_t *configFileName=canvas->fileAndDirName->GetOptionFileName();
	cfg->Load(configFileName);

	canvas->configDlg->Make(*cfg,canvas);
	canvas->AttachModalDialog(canvas->configDlg);

	canvas->SetNeedRedraw(YSTRUE);

	PolyCreConfig::Delete(cfg);
}

void FsGui3DMainCanvas::Option_ReapplyConfig(class PolyCreConfig &cfg)
{
	YsCoordSysModel=cfg.coordSys;
	drawEnv.RotateView(0.0,0.0);
	drawEnv.SetZoom(drawEnv.GetZoom());
	drawEnv.SetPerspective(cfg.perspective);

	this->config->defDataDir=cfg.defDataDir;

	SetNeedRedraw(YSTRUE);
}

void FsGui3DMainCanvas::Option_ReapplyConfig(void)
{
	const wchar_t *configFileName=fileAndDirName->GetOptionFileName();
	config->Load(configFileName);
	Option_ReapplyConfig(*config);
}
