#include "../fsgui3dapp.h"
#include "../miscdlg.h"
#include <ysgl.h>
#include <ysglbuffer.h>

#include <ysshellextmisc.h>
#include <ysshellext_geomcalc.h>
#include <ysshellext_trackingutil.h>
#include <ysshellextedit_localop.h>

#include "utildlg.h"


#include "../textresource.h"


/* static */ void FsGui3DMainCanvas::UtilMenu_ClearConstEdgeAll(void *appPtr,FsGuiPopUpMenu *,FsGuiPopUpMenuItem *)
{
	FsGui3DMainCanvas &canvas=*(FsGui3DMainCanvas *)appPtr;
	if(NULL==canvas.Slhd())
	{
		return;
	}

	YsShellExtEdit &shl=*(canvas.Slhd());
	YsShellExtEdit::StopIncUndo incUndo(shl);

	for(auto ceHd : shl.AllConstEdge())
	{
		shl.DeleteConstEdge(ceHd);
	}

	canvas.needRemakeDrawingBuffer|=(NEED_REMAKE_DRAWING_CONSTEDGE|NEED_REMAKE_DRAWING_SELECTED_CONSTEDGE);
	canvas.SetNeedRedraw(YSTRUE);
	if(YSTRUE==canvas.deletionRequireToClearUIMode)
	{
		canvas.Edit_ClearUIMode();
	}
}



/* static */ void FsGui3DMainCanvas::UtilMenu_ConstrainAllEdges(void *appPtr,FsGuiPopUpMenu *,FsGuiPopUpMenuItem *)
{
	FsGui3DMainCanvas &canvas=*(FsGui3DMainCanvas *)appPtr;
	if(NULL==canvas.Slhd())
	{
		return;
	}

	YsShellExtEdit &shl=*(canvas.Slhd());
	YsShellExtEdit::StopIncUndo incUndo(shl);

	YsShellEdgeEnumHandle edHd=NULL;
	while(YSOK==shl.MoveToNextEdge(edHd))
	{
		YsShellVertexHandle edVtHd[2];
		shl.GetEdge(edVtHd,edHd);

		if(0==shl.GetNumConstEdgeUsingEdgePiece(edVtHd))
		{
			shl.AddConstEdge(2,edVtHd,YSFALSE);
		}
	}

	canvas.needRemakeDrawingBuffer|=(NEED_REMAKE_DRAWING_CONSTEDGE|NEED_REMAKE_DRAWING_SELECTED_CONSTEDGE);
	canvas.SetNeedRedraw(YSTRUE);
	if(YSTRUE==canvas.deletionRequireToClearUIMode)
	{
		canvas.Edit_ClearUIMode();
	}
}



////////////////////////////////////////////////////////////



PolyCreAngleSliderDialog::PolyCreAngleSliderDialog()
{
	canvas=NULL;
	okBtn=NULL;
	cancelBtn=NULL;
}

PolyCreAngleSliderDialog::~PolyCreAngleSliderDialog()
{
}

/*static*/ PolyCreAngleSliderDialog *PolyCreAngleSliderDialog::Create(class FsGui3DMainCanvas *canvas)
{
	PolyCreAngleSliderDialog *dlg=new PolyCreAngleSliderDialog;
	dlg->canvas=canvas;
	return dlg;
}

/*static*/ void PolyCreAngleSliderDialog::Delete(PolyCreAngleSliderDialog *ptr)
{
	delete ptr;
}

YSRESULT PolyCreAngleSliderDialog::Make(void)
{
	if(NULL==okBtn)
	{
		FsGuiDialog::Initialize();

		slider=AddHorizontalSlider(0,FSKEY_NULL,40,0.0,180.0,YSTRUE);
		slider->SetPositionByScaledValue(60.0);
		angleTxt=AddTextBox(0,FSKEY_NULL,FsGuiTextBox::HORIZONTAL,FSGUI_COMMON_ANGLE,L"60",3,YSFALSE);

		okBtn=AddTextButton(0,FSKEY_ENTER,FSGUI_PUSHBUTTON,FSGUI_COMMON_OK,YSTRUE);
		cancelBtn=AddTextButton(0,FSKEY_ESC,FSGUI_PUSHBUTTON,FSGUI_COMMON_CANCEL,YSFALSE);

		SetArrangeType(FSDIALOG_ARRANGE_TOP_LEFT);
		SetBackgroundAlpha(0.3);
		Fit();

		RemakeDrawingBuffer();
	}

	return YSOK;
}

void PolyCreAngleSliderDialog::RemakeDrawingBuffer(void)
{
	lineBuf.CleanUp();

	YsShellExtEdit &shl=*(canvas->Slhd());

	const double angleThr=angleTxt->GetRealNumber();

	YsShellEdgeEnumHandle edHd=NULL;
	while(YSOK==shl.MoveToNextEdge(edHd))
	{
		YsShellVertexHandle edVtHd[2];
		shl.GetEdge(edVtHd,edHd);

		const double angleDeg=YsRadToDeg(YsShellExt_CalculateDihedralAngle(shl.Conv(),edVtHd));
		if(angleThr<=angleDeg)
		{
			lineBuf.AddVertex(shl.GetVertexPosition(edVtHd[0]));
			lineBuf.AddVertex(shl.GetVertexPosition(edVtHd[1]));
		}
	}
}

void PolyCreAngleSliderDialog::OnButtonClick(FsGuiButton *btn)
{
	if(btn==okBtn)
	{
		(*canvas->SpaceKeyCallBack)(*canvas);
		canvas->Edit_ClearUIMode();
	}
	else if(btn==cancelBtn)
	{
		canvas->Edit_ClearUIMode();
	}
}

void PolyCreAngleSliderDialog::OnTextBoxChange(FsGuiTextBox *txt)
{
	if(txt==angleTxt)
	{
		const double degree=txt->GetRealNumber();
		slider->SetPositionByScaledValue(degree);
		RemakeDrawingBuffer();
		canvas->SetNeedRedraw(YSTRUE);
	}
}

void PolyCreAngleSliderDialog::OnSliderPositionChange(FsGuiSlider *slider,const double &,const double &)
{
	if(slider==this->slider)
	{
		const double degree=slider->GetScaledValue();
		YsString str;
		str.Printf("%.1lf",degree);
		angleTxt->SetText(str);
		RemakeDrawingBuffer();
		canvas->SetNeedRedraw(YSTRUE);
	}
}



/*static*/ void FsGui3DMainCanvas::UtilMenu_AddConstEdgeByDihedralAngle(void *appPtr,FsGuiPopUpMenu *,FsGuiPopUpMenuItem *)
{
	FsGui3DMainCanvas &canvas=*(FsGui3DMainCanvas *)appPtr;
	if(NULL==canvas.Slhd())
	{
		return;
	}

	
	{
		if(YSOK==canvas.angleSliderDlg->Make())
		{
			canvas.Edit_ClearUIMode();

			canvas.AddDialog(canvas.angleSliderDlg);
			canvas.ArrangeDialog();

			canvas.UIDrawCallBack3D=UtilMenu_AddConstEdgeByDihedralAngle_DrawCallBack3D;
			canvas.SpaceKeyCallBack=UtilMenu_AddConstEdgeByDihedralAngle_SpaceKeyCallBack;

			canvas.undoRedoRequireToClearUIMode=YSFALSE;
			canvas.deletionRequireToClearUIMode=YSFALSE;

			canvas.SetNeedRedraw(YSTRUE);
		}
	}
}

/*static*/ void FsGui3DMainCanvas::UtilMenu_AddConstEdgeByDihedralAngle_DrawCallBack3D(FsGui3DMainCanvas &canvas)
{
	if(NULL==canvas.Slhd())
	{
		return;
	}

	auto renderer=YsGLSLSharedFlat3DRenderer();
	YsGLSLUse3DRenderer(renderer);

	auto &lineBuf=canvas.angleSliderDlg->lineBuf;

	glLineWidth(3);

	const GLfloat edgeCol[4]={1,0.3f,0.3f,1};
	YsGLSLSet3DRendererUniformColorfv(renderer,edgeCol);
	YsGLSLDrawPrimitiveVtxfv(renderer,GL_LINES,lineBuf.GetNumVertex(),lineBuf);

	glLineWidth(1);

	YsGLSLEndUse3DRenderer(renderer);
}

/*static*/ YSRESULT FsGui3DMainCanvas::UtilMenu_AddConstEdgeByDihedralAngle_SpaceKeyCallBack(FsGui3DMainCanvas &canvas)
{
	return canvas.UtilMenu_AddConstEdgeByDihedralAngle_SpaceKeyCallBack();
}

YSRESULT FsGui3DMainCanvas::UtilMenu_AddConstEdgeByDihedralAngle_SpaceKeyCallBack(void)
{
	if(NULL==Slhd())
	{
		return YSERR;
	}
	YsShellExtEdit &shl=*Slhd();



	YsShellExtEdit::StopIncUndo incUndo(shl);

	const double angleThr=angleSliderDlg->angleTxt->GetRealNumber();

	YsShellVertexStore usedVtx(shl.Conv());

	YsShellEdgeEnumHandle edHd=NULL;
	while(YSOK==shl.MoveToNextEdge(edHd))
	{
		YsShellVertexHandle edVtHd[2];
		shl.GetEdge(edVtHd,edHd);

		if(0==shl.GetNumConstEdgeUsingEdgePiece(edVtHd))
		{
			const double angleDeg=YsRadToDeg(YsShellExt_CalculateDihedralAngle(shl.Conv(),edVtHd));
			if(angleThr<=angleDeg)
			{
				shl.AddConstEdge(2,edVtHd,YSFALSE);
				usedVtx.AddVertex(edVtHd[0]);
				usedVtx.AddVertex(edVtHd[1]);
			}
		}
	}

	for(auto vtHd : usedVtx)
	{
		auto vtCeHd=shl.FindConstEdgeFromVertex(vtHd);
		if(2==vtCeHd.GetN() && vtCeHd[0]!=vtCeHd[1])
		{
			YsShellExtEdit_MergeConstEdge(shl,vtCeHd,YSTRUE);
		}
	}

	needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_CONSTEDGE;
	SetNeedRedraw(YSTRUE);

	Edit_ClearUIMode();

	return YSOK;
}


/* static */ void FsGui3DMainCanvas::UtilMenu_AddConstEdgeAlongNonManifoldEdge(void *appPtr,FsGuiPopUpMenu *,FsGuiPopUpMenuItem *)
{
	auto &canvas=*(FsGui3DMainCanvas *)appPtr;

	if(NULL==canvas.Slhd())
	{
		return;
	}
	YsShellExtEdit &shl=*canvas.Slhd();



	YsShellExtEdit::StopIncUndo incUndo(shl);

	YsShellVertexStore usedVtx(shl.Conv());

	YsShellEdgeEnumHandle edHd=NULL;
	while(YSOK==shl.MoveToNextEdge(edHd))
	{
		YsShellVertexHandle edVtHd[2];
		shl.GetEdge(edVtHd,edHd);

		if(0==shl.GetNumConstEdgeUsingEdgePiece(edVtHd))
		{
			if(2!=shl.GetNumPolygonUsingEdge(edVtHd))
			{
				shl.AddConstEdge(2,edVtHd,YSFALSE);
				usedVtx.AddVertex(edVtHd[0]);
				usedVtx.AddVertex(edVtHd[1]);
			}
		}
	}

	for(auto vtHd : usedVtx)
	{
		auto vtCeHd=shl.FindConstEdgeFromVertex(vtHd);
		if(2==vtCeHd.GetN() && vtCeHd[0]!=vtCeHd[1])
		{
			YsShellExtEdit_MergeConstEdge(shl,vtCeHd,YSTRUE);
		}
	}

	canvas.needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_CONSTEDGE;
	canvas.SetNeedRedraw(YSTRUE);

	canvas.Edit_ClearUIMode();
}


/* static */ void FsGui3DMainCanvas::UtilMenu_ClearFaceGroupAll(void *appPtr,FsGuiPopUpMenu *,FsGuiPopUpMenuItem *)
{
	FsGui3DMainCanvas &canvas=*(FsGui3DMainCanvas *)appPtr;
	if(NULL==canvas.Slhd())
	{
		return;
	}

	YsShellExtEdit &shl=*(canvas.Slhd());
	YsShellExtEdit::StopIncUndo incUndo(shl);

	for(auto fgHd : shl.AllFaceGroup())
	{
		shl.DeleteFaceGroup(fgHd);
	}

	canvas.needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_SELECTED_FACEGROUP;
	canvas.SetNeedRedraw(YSTRUE);
	if(YSTRUE==canvas.deletionRequireToClearUIMode)
	{
		canvas.Edit_ClearUIMode();
	}
}



/* static */ void FsGui3DMainCanvas::UtilMenu_MakeFaceGroupFromConstEdge(void *appPtr,FsGuiPopUpMenu *,FsGuiPopUpMenuItem *)
{
printf("%s %d\n",__FUNCTION__,__LINE__);
	FsGui3DMainCanvas &canvas=*(FsGui3DMainCanvas *)appPtr;
	if(NULL==canvas.Slhd())
	{
		return;
	}

	YsShellExtEdit &shl=*(canvas.Slhd());
	YsShellExtEdit::StopIncUndo incUndo(shl);

	class ConstEdgeBarrier : public YsShellExt_TrackingUtil::SearchCondition
	{
	public:
		virtual YSRESULT TestEdge(const YsShellExt &shl,YsShellVertexHandle edVtHd0,YsShellVertexHandle edVtHd1) const
		{
			if(0<shl.GetNumConstEdgeUsingEdgePiece(edVtHd0,edVtHd1))  // If the piece is used,
			{
				return YSERR;  // block it.
			}
			return YSOK;
		}
	};
	
	ConstEdgeBarrier barrier;

	for(auto fgHd : shl.AllFaceGroup())
	{
		shl.DeleteFaceGroup(fgHd);
	}


	YsShellPolygonStore visited(shl.Conv());
	for(auto seedPlHd : shl.AllPolygon())
	{
		if(YSTRUE!=visited.IsIncluded(seedPlHd))
		{
			auto plHdArray=YsShellExt_TrackingUtil::MakeEdgeConnectedPolygonGroup(shl.Conv(),seedPlHd,&barrier);
			shl.AddFaceGroup(plHdArray);

			for(auto plHd : plHdArray)
			{
				visited.AddPolygon(plHd);
			}
		}
	}

	canvas.needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_SELECTED_FACEGROUP;
	canvas.SetNeedRedraw(YSTRUE);
	canvas.Edit_ClearUIMode();
}


/* static */ void FsGui3DMainCanvas::UtilMenu_PaintBasedOnFaceGroup(void *appPtr,FsGuiPopUpMenu *,FsGuiPopUpMenuItem *)
{
	FsGui3DMainCanvas &canvas=*(FsGui3DMainCanvas *)appPtr;
	if(NULL==canvas.Slhd())
	{
		return;
	}

	YsShellExtEdit &shl=*(canvas.Slhd());
	YsShellExtEdit::StopIncUndo incUndo(shl);

	YsShellExt::FaceGroupStore painted(shl.Conv());

	YsColor fourteenColor[14];
	fourteenColor[ 0].SetIntRGB(0  ,0  ,255);
	fourteenColor[ 1].SetIntRGB(255,0  ,0  );
	fourteenColor[ 2].SetIntRGB(255,0  ,255);
	fourteenColor[ 3].SetIntRGB(0  ,255,0  );
	fourteenColor[ 4].SetIntRGB(0  ,255,255);
	fourteenColor[ 5].SetIntRGB(255,255,0  );
	fourteenColor[ 6].SetIntRGB(255,255,255);
	fourteenColor[ 7].SetIntRGB(0  ,0  ,128);
	fourteenColor[ 8].SetIntRGB(128,0  ,0  );
	fourteenColor[ 9].SetIntRGB(128,0  ,128);
	fourteenColor[10].SetIntRGB(0  ,128,0  );
	fourteenColor[11].SetIntRGB(0  ,128,128);
	fourteenColor[12].SetIntRGB(128,128,0  );
	fourteenColor[13].SetIntRGB(128,128,128);

	for(auto fgHd : shl.AllFaceGroup())
	{
		auto neiFgHdArray=shl.GetNeighborFaceGroup(fgHd);

		YsArray <YsColor> neiFgColor;
		for(auto neiFgHd : neiFgHdArray)
		{
			if(YSTRUE==painted.IsIncluded(neiFgHd))
			{
				auto neiFgPlHd=shl.GetFaceGroup(neiFgHd);
				if(0<neiFgPlHd.GetN())
				{
					neiFgColor.Append(shl.GetColor(neiFgPlHd[0]));
				}
			}
		}

		YSBOOL found=YSFALSE;
		YsColor foundColor;
		for(auto &col : fourteenColor)
		{
			if(YSTRUE!=neiFgColor.IsIncluded(col))
			{
				foundColor=col;
				found=YSTRUE;
				break;
			}
		}

		if(YSTRUE==found)
		{
			auto fgPlHd=shl.GetFaceGroup(fgHd);
			for(auto plHd : fgPlHd)
			{
				shl.SetPolygonColor(plHd,foundColor);
			}
			painted.AddFaceGroup(fgHd);
		}
		else
		{
			printf("!\n");
		}
	}

	canvas.needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_POLYGON;
	canvas.SetNeedRedraw(YSTRUE);
}
