#ifndef YSSHELLEXT_DIAGNOSEUTIL_IS_INCLUDED
#define YSSHELLEXT_DIAGNOSEUTIL_IS_INCLUDED
/* { */

#include <ysshellext.h>

class YsShellExt_DiagnoseUtil
{
public:
	class NonManifoldEdge
	{
	public:
		YsShellVertexHandle edVtHd[2];
		int nUsed;
	};
	class SelfIntersection
	{
	public:
		YsShellPolygonHandle plHd;
	};
	class NonPlanarPolygon
	{
	public:
		YsShellPolygonHandle plHd;
	};

private:
	int nThread;
	const YsShellExt *shl;

public:
	YsArray <NonManifoldEdge> nonManifoldEdgeArray;
	YsArray <SelfIntersection> selfIntersectionArray;
	YsArray <NonPlanarPolygon> nonPlanarPolygonArray;

	int debug_threadId; // For debugging.

	/*! Default constructore.  Nullify shl. */
	YsShellExt_DiagnoseUtil();

	/*! Cleans up the object. */
	void CleanUp(void);

	/*! Set number of threads. */
	void SetNumThread(int nThread);

	/*! This function sets the shell pointer to prepare for diagnostics. */
	void SetShell(const YsShellExt &shl);

	/*! Check for non-manifold edges and populates nonManifoldEdgeArray. */
	void CheckNonManifoldEdge(void);

	/*! Check for self-intersecting polygons and populates selfIntersectionArray. */
	void CheckSelfIntersection(void);
private:
	class CheckSelfIntersection_ThreadInfo;
	static void CheckSelfIntersection_ThreadEntry(CheckSelfIntersection_ThreadInfo *info);
	void CheckSelfIntersection_Thread(CheckSelfIntersection_ThreadInfo &info);

public:
	/*! Check for non-planar polygons and populates nonPlanarPolygonArray. */
	void CheckNonPlanarPolygon(void);
private:
	class CheckNonPlanrPolygon_ThreadInfo;
	void CheckNonPlanrPolygon_Thread(CheckNonPlanrPolygon_ThreadInfo &info);
};

/* } */
#endif
