#include <stdio.h>
#include <string.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xlocale.h>

#include <yssystemfont.h>

class YsSystemFontInternal
{
public:
	Display *dsp;
	Window rootWin;
	XFontSet fontSet;

	YsSystemFontInternal();
	~YsSystemFontInternal();
	void CleanUp(void);
	XFontSet MakeFontSet(int height);
	YSRESULT SetFontByHeight(int height);

	unsigned char *RGBABitmap(
		unsigned int &wid,unsigned int &hei,unsigned int &bytePerLine,
		const wchar_t str[],unsigned int outputBitPerPixel,
	    const unsigned char fgCol[3],const unsigned char bgCol[3],
	    YSBOOL reverse);

	YSRESULT GetTightBitmapSize(int &wid,int &hei,const wchar_t str[]) const;
};

YsSystemFontInternal::YsSystemFontInternal()
{
	const char *localeStr=setlocale(LC_CTYPE,"");
	if(NULL==localeStr || True!=XSupportsLocale())
	{
		setlocale(LC_CTYPE,"en_US.UTF-8");  // Default to en_US
	}

	dsp=NULL;
	rootWin=NULL;
	fontSet=NULL;

	dsp=XOpenDisplay(NULL);
	if(NULL!=dsp)
	{
		rootWin=DefaultRootWindow(dsp);
		fontSet=MakeFontSet(16);
	}
}

YsSystemFontInternal::~YsSystemFontInternal()
{
	if(NULL!=dsp)
	{
		CleanUp();
		XCloseDisplay(dsp);
	}
}

void YsSystemFontInternal::CleanUp(void)
{
	if(NULL!=fontSet)
	{
		XFreeFontSet(dsp,fontSet);
		fontSet=NULL;
	}
}

XFontSet YsSystemFontInternal::MakeFontSet(int height)
{
	if(NULL!=dsp)
	{
		const char *fontNameFmt=
			"-*-*-*-*-*-*-%d-*-*-*-*-*-iso8859-1,"
			"-*-*-*-*-*-*-%d-*-*-*-*-*-iso8859-2,"
			"-*-*-*-*-*-*-%d-*-*-*-*-*-iso8859-4,"
			"-*-*-*-*-*-*-%d-*-*-*-*-*-iso8859-5,"
			"-*-*-*-*-*-*-%d-*-*-*-*-*-iso8859-6,"
			"-*-*-*-*-*-*-%d-*-*-*-*-*-iso8859-7,"
			"-*-*-*-*-*-*-%d-*-*-*-*-*-iso8859-8,"
			"-*-*-*-*-*-*-%d-*-*-*-*-*-iso8859-9,"
			"-*-*-*-*-*-*-%d-*-*-*-*-*-iso8859-15,"
			"-*-*-*-*-*-*-%d-*-*-*-*-*-iso10646-1";
		char fontNameBuf[1024];
		snprintf(fontNameBuf,1023,fontNameFmt,
				 height,
				 height,
				 height,
				 height,
				 height,
				 height,
				 height,
				 height,
				 height,
				 height);
		fontNameBuf[1023]=0;
		
		int missSetCount;
		char *defStr,**missSet;
		return XCreateFontSet(dsp,fontNameBuf,&missSet,&missSetCount,&defStr);
	}
	return NULL;
}

YSRESULT YsSystemFontInternal::SetFontByHeight(int height)
{
	if(NULL!=dsp)
	{
		XFontSet newFontSet=MakeFontSet(height);
		if(NULL!=newFontSet)
		{
			CleanUp();
			fontSet=newFontSet;
			return YSOK;
		}
	}
	return YSERR;
}

unsigned char *YsSystemFontInternal::RGBABitmap(
	unsigned int &wid,unsigned int &hei,unsigned int &dstBytePerLine,
	const wchar_t str[],unsigned int dstBitPerPixel,
	const unsigned char fgCol[3],const unsigned char bgCol[3],
	YSBOOL reverse)
{
	if(NULL==str || 0==str[0] || (1!=dstBitPerPixel && 16!=dstBitPerPixel && 32!=dstBitPerPixel))
	{
		wid=0;
		hei=0;
		return NULL;
	}

	if(NULL!=dsp && NULL!=fontSet)
	{
		wid=0;
		hei=0;
		
		int i=0,i0=0;
		for(;;)
		{
			if(0==str[i] || '\n'==str[i])
			{
				XRectangle inkRect,logRect;
				const int ext=XwcTextExtents(fontSet,str+i0,i-i0,&inkRect,&logRect);
				
				if(wid<ext)
				{
					wid=ext;
				}
				hei+=logRect.height;
				i0=i+1;
			}
			if(0==str[i])
			{
				break;
			}
			i++;
		}
		
		wid=((wid+3)/4*4);
		if(0>=wid || 0>=hei)
		{
			wid=0;
			hei=0;
			return NULL;
		}

		const int defDepth=DefaultDepth(dsp,0);
		Pixmap pixmap=XCreatePixmap(dsp,rootWin,wid,hei,defDepth);
		GC gc;
		
		if(NULL!=(gc=XCreateGC(dsp,pixmap,0,0)))
		{
			XSetForeground(dsp,gc,BlackPixel(dsp,0));
			XFillRectangle(dsp,pixmap,gc,0,0,wid,hei);
		
			XSetForeground(dsp,gc,WhitePixel(dsp,0));
			int y=0;
			i=0;
			i0=0;
			for(;;)
			{
				if(0==str[i] || '\n'==str[i])
				{
					XRectangle inkRect,logRect;
					const int ext=XwcTextExtents(fontSet,str+i0,i-i0,&inkRect,&logRect);
					XwcDrawString(dsp,pixmap,fontSet,gc,0,y+logRect.height-1,str+i0,i-i0);

					y+=logRect.height;
					i0=i+1;
				}
				if(0==str[i])
				{
					break;
				}
				i++;
			}


			unsigned char *retBuf=NULL;
			XImage *image=XGetImage(dsp,pixmap,0,0,wid,hei,~0,XYPixmap);
			if(NULL!=image)
			{
				switch(dstBitPerPixel)
				{
				case 1:
					dstBytePerLine=((wid+31)&~31)/8;
					break;
				case 16:
				case 32:
					dstBytePerLine=wid*dstBitPerPixel/8;
					break;
				}

				const size_t bufSize=dstBytePerLine*hei;
				retBuf=new unsigned char [bufSize];

				if(1==dstBitPerPixel)
				{
					int i;
					for(i=0; i<bufSize; i++)
					{
						retBuf[i]=0;
					}
				}

				int x,y;
				unsigned char *dstLineTop=retBuf;
				for(y=0; y<image->height; y++)
				{
					const int yInImage=(YSTRUE!=reverse ? y : image->height-1-y);
					unsigned char srcMask=1;
					unsigned char *srcLineTop=(unsigned char *)image->data+yInImage*image->bytes_per_line;

					unsigned char dstMask=0x80;
					unsigned char *dstPtr=dstLineTop;

					for(x=0; x<image->width; x++)
					{
						const unsigned char *col;
						unsigned char alpha;
						if(0!=((*srcLineTop)&srcMask))
						{
							col=fgCol;
							alpha=255;
						}
						else
						{
							col=bgCol;
							alpha=0;
						}

						switch(dstBitPerPixel)
						{
						case 1:
							if(255==alpha)
							{
								(*dstPtr)|=dstMask;
							}

							if(1==dstMask)
							{
								dstMask=0x80;
								dstPtr++;
							}
							else
							{
								dstMask>>=1;
							}
							break;
						case 16:
							dstLineTop[x*2  ]=col[0];
							dstLineTop[x*2+1]=alpha;
							break;
						case 32:
							dstLineTop[x*4  ]=col[0];
							dstLineTop[x*4+1]=col[1];
							dstLineTop[x*4+2]=col[2];
							dstLineTop[x*4+3]=alpha;
							break;
						}
						if(0x80==srcMask)
						{
							srcMask=1;
							srcLineTop++;
						}
						else
						{
							srcMask<<=1;
						}
					}
					for(x=x; x<wid; x++)
					{
						switch(dstBitPerPixel)
						{
						case 16:
							dstLineTop[x*2  ]=bgCol[0];
							dstLineTop[x*2+1]=0;
							break;
						case 32:
							dstLineTop[x*4  ]=bgCol[0];
							dstLineTop[x*4+1]=bgCol[1];
							dstLineTop[x*4+2]=bgCol[2];
							dstLineTop[x*4+3]=0;
							break;
						}
					}
					dstLineTop+=dstBytePerLine;
				}
			}

			XFreeGC(dsp,gc);
			XFreePixmap(dsp,pixmap);
			return retBuf;
		}
	}

	return NULL;
}

YSRESULT YsSystemFontInternal::GetTightBitmapSize(int &wid,int &hei,const wchar_t str[]) const
{
	if(NULL==str || 0==str[0])
	{
		wid=0;
		hei=0;
		return YSOK;
	}

	if(NULL!=dsp && NULL!=fontSet)
	{
		wid=0;
		hei=0;

		int i=0,i0=0;
		for(;;)
		{
			if(0==str[i] || '\n'==str[i])
			{
				XRectangle inkRect,logRect;
				const int ext=XwcTextExtents(fontSet,str+i0,i-i0,&inkRect,&logRect);
				
				if(wid<ext)
				{
					wid=ext;
				}
				hei+=logRect.height;
				i0=i+1;
			}
			if(0==str[i])
			{
				break;
			}
			i++;
		}

		if(0>=wid || 0>=hei)
		{
			wid=0;
			hei=0;
			return YSERR;
		}
	}
	return YSOK;
}

YsSystemFontCache::YsSystemFontCache()
{
	internal=new YsSystemFontInternal();
}

YsSystemFontCache::~YsSystemFontCache()
{
	delete internal;
}

YSRESULT YsSystemFontCache::RequestDefaultFont(void)
{
	return internal->SetFontByHeight(16);
}

YSRESULT YsSystemFontCache::RequestDefaultFontWithHeight(int height)
{
	return internal->SetFontByHeight(height);
}

YSRESULT YsSystemFontCache::MakeSingleBitBitmap(YsSystemFontTextBitmap &bmp,const wchar_t wStr[],YSBOOL reverse) const
{
	unsigned int wid,hei,bytePerLine;
	const unsigned int bitPerPixel=1;
	const unsigned char fgCol[3]={255,255,255},bgCol[3]={0,0,0};

	unsigned char *dat=internal->RGBABitmap(wid,hei,bytePerLine,wStr,bitPerPixel,fgCol,bgCol,reverse);
	if(NULL!=dat)
	{
		bmp.SetBitmap(wid,hei,bytePerLine,bitPerPixel,dat);
		return YSOK;
	}
	return YSERR;
}

YSRESULT YsSystemFontCache::MakeRGBABitmap(YsSystemFontTextBitmap &bmp,const wchar_t wStr[],const unsigned char fgCol[3],const unsigned char bgCol[3],YSBOOL reverse) const
{
	unsigned int wid,hei,bytePerLine;
	const unsigned int bitPerPixel=32;

	unsigned char *dat=internal->RGBABitmap(wid,hei,bytePerLine,wStr,bitPerPixel,fgCol,bgCol,reverse);
	if(NULL!=dat)
	{
		bmp.SetBitmap(wid,hei,bytePerLine,bitPerPixel,dat);
		return YSOK;
	}
	return YSERR;
}

YSRESULT YsSystemFontCache::MakeGrayScaleAndAlphaBitmap(YsSystemFontTextBitmap &bmp,const wchar_t wStr[],unsigned char fgColS,unsigned char bgColS,YSBOOL reverse) const
{
	const unsigned char fgCol[3]={fgColS,fgColS,fgColS},bgCol[3]={bgColS,bgColS,bgColS};

	unsigned int wid,hei,bytePerLine;
	const unsigned int bitPerPixel=16;

	unsigned char *dat=internal->RGBABitmap(wid,hei,bytePerLine,wStr,bitPerPixel,fgCol,bgCol,reverse);
	if(NULL!=dat)
	{
		bmp.SetBitmap(wid,hei,bytePerLine,bitPerPixel,dat);
		return YSOK;
	}
	return YSERR;
}

YSRESULT YsSystemFontCache::GetTightBitmapSize(int &wid,int &hei,const wchar_t str[]) const
{
	return internal->GetTightBitmapSize(wid,hei,str);
}
